#!/bin/sh
#-------------------------------------------------------------------
# config.sh: This file is read at the beginning of the execution of the ASGS to
# set up the runs  that follow. It is reread at the beginning of every cycle,
# every time it polls the datasource for a new advisory. This gives the user
# the opportunity to edit this file mid-storm to change config parameters
# (e.g., the name of the queue to submit to, the addresses on the mailing list,
# etc)
#-------------------------------------------------------------------
#
# Copyright(C) 2019 Jason Fleming
#
# This file is part of the ADCIRC Surge Guidance System (ASGS).
#
# The ASGS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
## ASGS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# the ASGS.  If not, see <http://www.gnu.org/licenses/>.
#-------------------------------------------------------------------

# Fundamental

INSTANCENAME=custom-intel-unity       #adcirc-cg-001      # "name" of this ASGS process
#SCRATCHDIR=/projects/ees/dhs-crc/joshua_p/asgs/${INSTANCENAME}
RMQMessaging_Transmit=off             # JWP: valuable on hatteras, but no point so far on Unity
QSCRIPTTEMPLATE=$SCRIPTDIR/qscript.template.jwp # Same as qscript.template except with Kevin's recommended sbatch flags
#ADCIRCDIR=/home/joshua_port_uri_edu/adcirc-cg/work  # Dave Ullman's modified version of ADCIRC, built with ASGS's NetCDF libraries 
HOTSTARTFORMAT=netcdf              # added 1/28/23 to avoid hotstart-related errors

# Input files and templates

GRIDNAME=ricv1_noriv_nopump # ricv1, ricv1_noriv, ricv1_noriv_nopump
#GRIDNAME=ec95d #hsofs # Just while testing; once I'm sure it's good, we can go back to the URI mesh
source $SCRIPTDIR/config/mesh_defaults.sh

GFSBACKDIR="/pub/data/nccf/com/gfs/v16.3" # See StormSurgeLive/asgs Discussion #1064; will be fixed soon, but necessary as a workaround 2023-01-05

# Initial state (overridden by STATEFILE after ASGS gets going)

COLDSTARTDATE=2021081218  # calendar year month day hour YYYYMMDDHH24
HOTORCOLD=coldstart       # "hotstart" or "coldstart"
LASTSUBDIR=null

TIMEVARYINGWEIR=on
WEIRCLOSEDATE=2021081218
#WEIRCLOSEDATE=2021082206
hooksScripts[BUILD_NOWCAST_SCENARIO]="generate_tvwfile.sh" # uncomment this when running with TIMEVARYINGWEIR=on
hooksScripts[BUILD_FORECAST_SCENARIO]="generate_tvwfile.sh" # uncomment this when running with TIMERVARYINGWEIR=on

# Physical forcing (defaults set in config/forcing_defaults.sh)

TIDEFAC=on               # tide factor recalc
   HINDCASTLENGTH=7.0   # length of initial hindcast, from cold (days)
BACKGROUNDMET=off        # wind download/forcing; set to "on" or "NAM" for NAM
   #FORECASTCYCLE="00,06,12,18"
   FORECASTCYCLE="00"    # reduced cadence to be mindful of CPU time while testing
TROPICALCYCLONE=on      # tropical cyclone forcing
   STORM=08              # storm number, e.g. 05=ernesto in 2006
   YEAR=2021             # year of the storm
   TRIGGER=rssembedded
   RSSSITE=filesystem
   FTPSITE=filesystem
   HDIR=/home/joshua_port_uri_edu/asgs-fork/henri_test
   FDIR=/home/joshua_port_uri_edu/asgs-fork/henri_test
WAVES=off                # wave forcing
   REINITIALIZESWAN=off  # used to bounce the wave solution
VARFLUX=off              # variable river flux forcing
   RIVERSITE=filesystem
   RIVERDIR=/home/joshua_port_uri_edu/asgs-fork/riverdata
   RIVERUSER=joshua_port_uri_edu
   RIVERDATAPROTOCOL=filesystem
CYCLETIMELIMIT="99:00:00"

# Computational Resources (related defaults set in platforms.sh)

# I'm attempting to manually request exlusive nodes in qscript.template.jwp
# $nnodes = int($totalcpu/$ppn) ROUNDED UP, per qscript.pl. So, using PPN I should be able to get the right amount of nodes. e.g. NCPU=127, NUMWRITERS=1, PPN=64 gives 128 for $totalcpu and 2 for $nnodes, which will be exclusive. It should be guaranteed exclusive if we add the exclusive flag to qscript.template.jwp too.
PPN=64
NCPU=511 #511 255 127                # number of compute CPUs for all simulations
NCPUCAPACITY=512 #512 256 128        # NCPU + NUMWRITERS for jobs running in series
NUMWRITERS=1 # ASGS will request a number of CPUs equal to NCPU+NUMWRITERS; Brett recommends NUMWRITERS=1 in most cases
ACCOUNT=null
#QUEUENAME="uri-cpu"
QUEUENAME=uri-richamp #uri-cpu # same as SLURM partition
SERQUEUE=uri-richamp
RESERVATION=richamp


# Post processing and publication

INTENDEDAUDIENCE=developers-only    # "general" | "developers-only" | "professional"

# JWP 3/30/23 commented all of the below out because THREDDS isn't set up
# All we need is richamp_scale_and_subset, plus whatever job posts the RICHAMP* files
#OPENDAPPOST=opendap_post2.sh

#FINISH_NOWCAST_SCENARIO=( output/opendap_post_nowcast.sh ) # output/run_adda.sh )
#POSTPROCESS=( richamp-support/richamp_scale_and_subset.sh createOPeNDAPFileList.sh $OPENDAPPOST )
#POSTPROCESS=( richamp-support/richamp_scale_and_subset.sh ) # TURN THIS ONE BACK ON AFTER TABLETOP RUNS
#postAdditionalFiles=( RICHAMP_fort63.nc RICHAMP_wind.nc )

#TDS=(renci_tds) # was blank in run.properties, at least at first; unsure of impact of omitting this
#TARGET=hatteras # was blank in run.properties, at least at first; unsure of impact of omitting this
#OPENDAPNOTIFY="joshua_port@uri.edu"
#NOTIFY_SCRIPT=ncfs_nam_notify.sh
#NOTIFY_SCRIPT=null_notify.sh
#NOTIFY_SCRIPT=blanton_cyclone_notify.sh

HINDCASTWALLTIME="03:00:00" # hindcast wall clock time
ADCPREPWALLTIME="01:00:00"  # adcprep wall clock time, including partmesh
NOWCASTWALLTIME="03:00:00"  # longest nowcast wall clock time
FORECASTWALLTIME="03:00:00" # forecast wall clock time

# Scenario package

#PERCENT=default
SCENARIOPACKAGESIZE=1 # setting to 1-10 makes ASGS generate 1-10 forecasts too. 0 is just nowcasts.
case $si in
   -2) 
       ENSTORM=hindcast
       ;;
   -1)      
       # do nothing ... this is not a forecast
       ENSTORM=nowcast
       ;;
    0)
       ENSTORM=forecast  # was namforecast; updated in case the product name is a keyword...and later updated to forecast to remove confusion when running in TC mode
       ;;
    1)
       ENSTORM=veerLeftEdge
       PERCENT=-100 # -100 for left edge, 100 for right edge
       ;;
    2) ENSTORM=veerRightEdge
       PERCENT=100
       ;;
    *)   
       echo "CONFIGRATION ERROR: Unknown ensemble member number: '$si'."
      ;;
esac

PREPPEDARCHIVE=prepped_${GRIDNAME}_${INSTANCENAME}_${NCPU}.tar.gz
HINDCASTARCHIVE=prepped_${GRIDNAME}_hc_${INSTANCENAME}_${NCPU}.tar.gz
